package lightcontainer.protocol;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class ProtocolReader {
    // Variables
    private final Pattern rulePattern;

    // Constructor
    protected ProtocolReader(String pattern) {
        this.rulePattern = Pattern.compile(pattern);
    }

    public enum ResultCmdReceiver {
        CLIENT,
        STOREBACKEND
    }

    public abstract class ProtocolResult {

        /**
         * Command qui sera renvoyée par exemple au client
         */
        private String resultCommand;
        /**
         * Désigne vers ou cette commande est envoyée.
         * ResultCmdReceiver.CLIENT : Signifie que cette commande va être directement revoyée au client.
         * ResultCmdReceiver.STOREBACKEND : Signifie que cette commande va être envoyée dans la file de tâche du server et êre en attente d'envoie à un StorBackEnd
         */
        private ResultCmdReceiver receiver;


        public ResultCmdReceiver getReceiver() {
            return receiver;
        }

        public String getResultCommand() {
            return resultCommand;
        }

        public void setResultCommand(String resultCommand, ResultCmdReceiver receiver) {
            this.resultCommand = resultCommand;
            this.receiver = receiver;
        }
    }

    /**
     * Permet de lancer la décomposition d'une commande pour en extraire les données
     * @param data Contenu de la commande
     */
    public <T extends ProtocolResult> T execute(String data) {
        Matcher ruleMatcher = this.rulePattern.matcher(data);

        if (ruleMatcher.matches()) {
            String[] groups = new String[ruleMatcher.groupCount()];

            for (int i = 1; i <= groups.length; ++i)
                groups[i - 1] = ruleMatcher.group(i);

            return onExecuted(groups);
        }
        return null;
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     * @param data Paramètres pour créer la commande.
     */
    protected abstract <T> T onExecuted(String... data);
}
